unit SearchMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Mask, ClientMain, ComCtrls, Buttons;

type
  TfrmSearchMain = class(TForm)
    meValue: TMaskEdit;
    lblFieldCaption: TLabel;
    cbxRelations: TComboBox;
    meRangeHigh: TMaskEdit;
    dtpFrom: TDateTimePicker;
    dtpTo: TDateTimePicker;
    btnOK: TBitBtn;
    btnCancel: TBitBtn;
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure meValueKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure meValueChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure meValueKeyPress(Sender: TObject; var Key: Char);
    procedure cbxRelationsChange(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure meValueDblClick(Sender: TObject);
  private
    { Private declarations }
    NotInSearch:boolean;
  public
    { Public declarations }
    FLocated:boolean;
    FSearchListDisplayed:boolean;
    FDisplayField:string;
    FFieldType:integer;
    function LocateText(Sender: TObject;ADisplayField:string):boolean;
  end;

var
  frmSearchMain: TfrmSearchMain;

implementation

{$R *.DFM}
uses DB, SearchList, ClientMainDataModule;

procedure TfrmSearchMain.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key=VK_RETURN) and (FLocated or (cbxRelations.Text=''))then
    ModalResult:=mrOK
  else
  if not FSearchListDisplayed and ((Key=VK_ESCAPE) or ((Key=VK_RETURN) and not FLocated and (meValue.EditText=''))) then
    ModalResult:=mrCancel;
end;

procedure TfrmSearchMain.meValueKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (FFieldType=varString) and (Key=VK_BACK) then
  with Sender as TCustomEdit do
  Begin
    if (SelStart>0) and (SelLength>0) then
    Begin
      SelStart:=SelStart-1;
      SelLength:=Length(Text)-SelStart;
    End;
  End
  else
  if (ssCtrl in Shift) and (Key=VK_SPACE) and (FDisplayField<>'') then
  with frmSearchList do
  Begin
    frmSearchList.NotInSearch:=False;
    edtSearch.Text:=meValue.Text;
    FSelStart:=meValue.SelStart;
    FSelLength:=meValue.SelLength;
    frmSearchList.NotInSearch:=True;
    FLocated:=frmSearchMain.FLocated;
    FDisplayField:=frmSearchMain.FDisplayField;
    FSearchListDisplayed:=True;
    if ShowModal=mrOK then
    Begin
      frmSearchMain.NotInSearch:=False;
      meValue.Text:=edtSearch.Text;
      meValue.SelStart:=edtSearch.SelStart;
      meValue.SelLength:=edtSearch.SelLength;
      frmSearchMain.NotInSearch:=True;
      frmSearchMain.FLocated:=FLocated;
    End
    else
      FLocated:=LocateText(meValue,frmSearchMain.FDisplayField);
    FSearchListDisplayed:=false;
  End;
end;

procedure TfrmSearchMain.meValueChange(Sender: TObject);
begin
  if TWinControl(Sender).Focused and dmClientMain.dsSearch.Active and NotInSearch and (FDisplayField<>'') then
  try
    NotInSearch:=False;
    FLocated:=LocateText(Sender,FDisplayField);
    NotInSearch:=True;
  except
    dmClientMain.dsSearch.Close;
    NotInSearch:=True;
    FLocated:=False;
  end;
end;

procedure TfrmSearchMain.FormShow(Sender: TObject);
begin
  NotInSearch:=True;
  FLocated:=FDisplayField='';
  if (FDisplayField<>'') and (meValue.Text<>'') then
    FLocated:=LocateText(meValue,FDisplayField);
  cbxRelationsChange(cbxRelations);
end;

procedure TfrmSearchMain.meValueKeyPress(Sender: TObject; var Key: Char);
begin
  if FSearchListDisplayed then
    key:=#0
  else
  if (FFieldType=varInteger) and
    not(Key in ['0'..'9',#8]) then
    key:=#0
end;

procedure TfrmSearchMain.cbxRelationsChange(Sender: TObject);
begin
  if (FFieldType=varDate) then
  Begin
    dtpTo.Enabled:=cbxRelations.Text='><';
    dtpFrom.Enabled:=(cbxRelations.Text<>'') and (cbxRelations.Text<>'!');
    meValue.Width:=meRangeHigh.Width;
  End
  else
  Begin
    if (cbxRelations.Text='><') then
      meValue.Width:=meRangeHigh.Width
    else
      meValue.Width:=(meRangeHigh.Left-meValue.Left) + meRangeHigh.Width;
    meValue.Enabled:=(cbxRelations.Text<>'') and (cbxRelations.Text<>'!');
    meRangeHigh.Enabled:=cbxRelations.Text='><';
  End;
end;

function TfrmSearchMain.LocateText(Sender: TObject;ADisplayField:string):boolean;
var lvSelStart:integer;
begin
  Result:=dmClientMain.dsSearch.Locate(ADisplayField,TCustomEdit(Sender).Text,[loCaseInsensitive,loPartialKey]);
  if Result then
  Begin
    lvSelStart:=TCustomEdit(Sender).SelStart;
    TCustomEdit(Sender).Text:=trim(dmClientMain.dsSearch.FieldByName(ADisplayField).asString);
    TCustomEdit(Sender).SelStart:=lvSelStart;
    TCustomEdit(Sender).SelLength:=Length(TCustomEdit(Sender).Text)-lvSelStart;
  End;
end;

procedure TfrmSearchMain.btnOKClick(Sender: TObject);
begin
  ModalResult:=mrOK;
end;

procedure TfrmSearchMain.btnCancelClick(Sender: TObject);
begin
  ModalResult:=mrCancel;
end;

procedure TfrmSearchMain.meValueDblClick(Sender: TObject);
var Key:word;
begin
  Key:=VK_SPACE;
  meValueKeyDown(meValue,Key,[ssCtrl]);
end;

end.
